@@ -49,18 +49,25 @@
 
 package com.openexchange.filestore.sproxyd.osgi;
 
+import java.rmi.Remote;
+import java.util.Dictionary;
+import java.util.Hashtable;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.context.ContextService;
 import com.openexchange.database.CreateTableService;
 import com.openexchange.database.DatabaseService;
+import com.openexchange.filestore.FileStorageProvider;
 import com.openexchange.filestore.sproxyd.SproxydFileStorageFactory;
 import com.openexchange.filestore.sproxyd.groupware.SproxydCreateTableService;
 import com.openexchange.filestore.sproxyd.groupware.SproxydCreateTableTask;
 import com.openexchange.filestore.sproxyd.groupware.SproxydDeleteListener;
+import com.openexchange.filestore.sproxyd.rmi.SproxydRemoteManagement;
+import com.openexchange.filestore.sproxyd.rmi.impl.SproxydRemoteImpl;
 import com.openexchange.groupware.delete.DeleteListener;
 import com.openexchange.groupware.update.DefaultUpdateTaskProviderService;
 import com.openexchange.groupware.update.UpdateTaskProviderService;
 import com.openexchange.osgi.HousekeepingActivator;
-import com.openexchange.tools.file.external.FileStorageFactoryCandidate;
+import com.openexchange.timer.TimerService;
 
 /**
  * {@link SproxydActivator}
@@ -80,20 +87,31 @@ public class SproxydActivator extends HousekeepingActivator {
 
     @Override
     protected Class<?>[] getNeededServices() {
-        return new Class<?>[] { ConfigurationService.class, DatabaseService.class };
+        return new Class<?>[] { ConfigurationService.class, DatabaseService.class, TimerService.class };
     }
 
     @Override
     protected void startBundle() throws Exception {
         LOG.info("Starting bundle: com.openexchange.filestore.sproxyd");
 
+        // Trackers
+        trackService(ContextService.class);
+        openTrackers();
+
         // Register update task, create table job and delete listener
         registerService(CreateTableService.class, new SproxydCreateTableService());
         registerService(UpdateTaskProviderService.class, new DefaultUpdateTaskProviderService(new SproxydCreateTableTask(this)));
         registerService(DeleteListener.class, new SproxydDeleteListener());
 
         // Register factory
-        registerService(FileStorageFactoryCandidate.class, new SproxydFileStorageFactory(this));
+        registerService(FileStorageProvider.class, new SproxydFileStorageFactory(this));
+
+        // Register RMI
+        {
+            Dictionary<String, Object> props = new Hashtable<String, Object>(2);
+            props.put("RMIName", SproxydRemoteManagement.RMI_NAME);
+            registerService(Remote.class, new SproxydRemoteImpl(this), props);
+        }
     }
 
     @Override
